import os
import sys

if __name__ == "__main__":
    os.environ.setdefault(
        "DJANGO_SETTINGS_MODULE", "pg_prefix_search.scripts.mock_settings"
    )
    from django.core.management import execute_from_command_line

    args = sys.argv + ["makemigrations"]
    execute_from_command_line(args)



from django.db.models import QuerySet, Q
from unidecode import unidecode

from pg_prefix_search.models import ImmutableUnaccent


def apply_search_by_name(queryset, field_name, query_text) -> QuerySet:
    """
    Builds a correct fill-text search query part:
    - allows searching by word beginnings ("rej kra gosp" -> "Sąd Rejonowy w Krakowie wydział Gospodarczy")
    - making sure that DB index is used
    - handling polish accent letters and upper/lower case
    """
    queryset = queryset.annotate(search_field_unaccented=ImmutableUnaccent(field_name))
    q = build_search_query_by_word_beginnings(query_text)
    return queryset.filter(q)


def build_search_query_by_word_beginnings(query_text) -> Q:
    """Builds a Q object that searches by word beginnings ("rej kra gosp" -> "Sąd Rejonowy w Krakowie wydział Gospodarczy")"""

    q = Q()
    if not query_text:
        return q

    unaccented_term = unidecode(query_text)

    for word in unaccented_term.split(' '):
        if word:
            q = q & (
                Q(search_field_unaccented__istartswith=word) |
                Q(search_field_unaccented__icontains=' '+word)
            )

    return q



from django.db import connection
from django.db import migrations


def create_extension(ext_name):
    """Checking if Postgres extension is installed"""
    with connection.cursor() as cursor:
        cursor.execute(f"SELECT * FROM pg_catalog.pg_extension WHERE extname='{ext_name}'")
        res = cursor.fetchone()
        if res is None:
            cursor.execute(f"CREATE EXTENSION {ext_name}")


def enable_extensions(apps, schema_editor):
    create_extension('pg_trgm')
    create_extension('unaccent')


def create_imm_unaccent_stored_proc(apps, schema_editor):
    """
    This stored procedure is needed to build GIN index on unaccented words,
    to allow searching without regards for accents (a=ą)
    """

    with connection.cursor() as cursor:
        cursor.execute("""
            CREATE OR REPLACE FUNCTION imm_unaccent(text) RETURNS text AS
            $$
            SELECT public.unaccent('public.unaccent', $1)
            $$ LANGUAGE sql IMMUTABLE;
        """)


class Migration(migrations.Migration):
    dependencies = []
    operations = [
        migrations.RunPython(enable_extensions),
        migrations.RunPython(create_imm_unaccent_stored_proc),
    ]



import os

SECRET_KEY = 'Used only for tests'
DEBUG = True

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql",
        "NAME": "postgres",
        "USER": "postgres",
        "PASSWORD": "example",
        "HOST": "localhost",
        "PORT": "5432",
    }
}

INSTALLED_APPS = [
    # Django apps necessary to run the admin site
    'django.contrib.auth',
    'django.contrib.admin',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.staticfiles',

    # And this app
    'pg_prefix_search',
]

STATIC_URL = '/static/'
MIDDLEWARE = [
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
]

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'OPTIONS': {
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
            ],
        }
    },
]



from django.db import models
from django.db.models.functions import Upper
from django.contrib.postgres.indexes import GinIndex, OpClass


class ImmutableUnaccent(models.Func):
    function = 'imm_unaccent'


def text_searchable_index_spec(field_name):
    """
    Specification of GIN index for a column that should be full-text searchable.
    This exact index format is important, the following contains benchmarks that prove it:
    https://gist.github.com/ssobczak/572fdda8ad7aef91abe968c5fcc755ff
    """
    return OpClass(Upper(ImmutableUnaccent(field_name)), 'gin_trgm_ops')


class SearchableModel(models.Model):
    name = models.CharField('Nazwa', max_length=1024)

    class Meta:
        abstract = True

        indexes = (
            GinIndex(text_searchable_index_spec('name'), name='searchable_model_name__gin'),
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pg_prefix_search', 'pg_prefix_search.migrations', 'pg_prefix_search.scripts']

package_data = \
{'': ['*']}

install_requires = \
['Django>=3.2,<4.0', 'Unidecode>=1.3.4,<2.0.0']

setup_kwargs = {
    'name': 'pg-prefix-search',
    'version': '0.1.0',
    'description': 'Text search by word fragments using Postgres-specific indices.',
    'long_description': None,
    'author': 'Szymon Sobczak',
    'author_email': 'szymon@solvbot.pl',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
